"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
/** @module stately-react */
var React = require("react");
var Async_1 = require("./Async");
/**
 * Props: {@link CallableAsyncProps} extends Omit<{@link AsyncProps}, 'children' | 'params'>
 *
 * A render-prop component that injects an `AsyncLifecycle` for any `AsyncOperation` into the component tree.
 * Identical to {@link Async}, except that instead of accepting `params` as a prop, it passes `call(...params: Params)` to its [children]{@link CallableAsyncProps}.
 * As such, `CallableAsync` gives the `children` control over when the operation is executed - typically as a result of a user interaction.
 * This can be used for user-initiated asynchronous calls, such as saving a form.
 *
 * The following example invokes `save` when the button is clicked:
 * ```
 * // type save = (entity: Entity) => Promise
 *
 * <CallableAsync operation={save}>
 *   {(state, call) =>
 *     <div>
 *       {
 *         // If the Promise is rejected, render the error.
 *         state.error ? <span className="error">{state.error}</span>
 *           // If the Promise resolves, render a success message.
 *           : state.status === 'complete' ? <span className="success">Entity saved successfully.</span>
 *           // If the operation is active, render a loading indicator.
 *           : state.status === 'active' && <span className="loading" />
 *       }
 *       <button
 *         onClick={() => save(entity)}>
 *         Save
 *       </button>
 *     </div>}
 * </CallableAsync>
 * ```
 *
 * The implementation of `CallableAsync` is trivial. It is a stateful wrapper around `Async` that calls `setState(params)` when `call(params)` is invoked.
 * More complex use cases that are not met by `Async` or `CallableAsync` can be achieved using a similar strategy.
 */
var CallableAsync = /** @class */ (function (_super) {
    __extends(CallableAsync, _super);
    function CallableAsync() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {};
        _this.call = function () {
            var params = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                params[_i] = arguments[_i];
            }
            _this.setState({ params: params });
        };
        return _this;
    }
    CallableAsync.prototype.render = function () {
        var _this = this;
        var _a = this.props, children = _a.children, props = __rest(_a, ["children"]);
        var params = this.state.params;
        return (React.createElement(Async_1.Async, __assign({}, props, { params: params }), function (state) { return children(state, _this.call); }));
    };
    return CallableAsync;
}(React.Component));
exports.CallableAsync = CallableAsync;
//# sourceMappingURL=CallableAsync.js.map