"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
/** @module stately-react */
var React = require("react");
var observables_1 = require("stately-async/observables");
exports.createStoreContext = function (store) {
    return createSubjectContext(observables_1.$fromStore(store), store.getState());
};
function createSubjectContext(subject, initial) {
    var _a = React.createContext(undefined), Provider = _a.Provider, Consumer = _a.Consumer;
    var subjectNext = subject.next.bind(subject);
    var Subscription = /** @class */ (function (_super) {
        __extends(Subscription, _super);
        function Subscription() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this.state = { value: initial };
            _this.subscription = null;
            _this.onNext = function (value) {
                _this.setState({ value: value });
            };
            return _this;
        }
        Subscription.prototype.render = function () {
            var value = this.state.value;
            var children = this.props.children;
            return (React.createElement(Provider, { value: value }, typeof children === 'function'
                ? children(value, subjectNext)
                : children));
        };
        Subscription.prototype.componentDidMount = function () {
            this.subscription = subject.subscribe(this.onNext);
        };
        Subscription.prototype.componentWillUnmount = function () {
            this.subscription.unsubscribe();
        };
        return Subscription;
    }(React.Component));
    var Subscriber = /** @class */ (function (_super) {
        __extends(Subscriber, _super);
        function Subscriber() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Subscriber.prototype.render = function () {
            var _this = this;
            return (React.createElement(Consumer, null, function (state) {
                return typeof state === 'undefined'
                    ? warnNoProvider()
                    : _this.props.children(state, subjectNext);
            }));
        };
        return Subscriber;
    }(React.Component));
    function subscriber(deriveProps) {
        return function (Component) {
            var SubscriberComponent = /** @class */ (function (_super) {
                __extends(SubscriberComponent, _super);
                function SubscriberComponent() {
                    return _super !== null && _super.apply(this, arguments) || this;
                }
                SubscriberComponent.prototype.render = function () {
                    var _this = this;
                    return (React.createElement(Consumer, null, function (state) {
                        return typeof state === 'undefined' ? (warnNoProvider()) : (React.createElement(Component, __assign({}, _this.props, deriveProps(state, subjectNext))));
                    }));
                };
                return SubscriberComponent;
            }(React.Component));
            ;
            SubscriberComponent.displayName = "subscriber(" + (Component.name || Component.displayName) + ")";
            return SubscriberComponent;
        };
    }
    return {
        Subscription: Subscription,
        Subscriber: Subscriber,
        subscriber: subscriber,
    };
}
exports.createSubjectContext = createSubjectContext;
var warnNoProvider = function () {
    console.warn('stately-react:\n', '<Subscriber> was used with no ancestral <Subscription>.', 'Without a Subscription, a Subscriber will never receive data, and will therefore never render.\n', 'Make sure that there is a Subscription component in the component tree above the Subscriber.');
    return null;
};
//# sourceMappingURL=Subscribable.js.map